var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { Animated, Easing, StyleSheet, View, } from 'react-native';
import React, { useCallback, useContext, useMemo, useRef, useState, } from 'react';
const styles = StyleSheet.create({
    container: {
        alignItems: 'center',
        bottom: 0,
        flex: 1,
        justifyContent: 'center',
        left: 0,
        position: 'absolute',
        right: 0,
        top: 0,
    },
    iconContainer: {
        alignItems: 'center',
        justifyContent: 'center',
    },
    view: {
        flex: 1,
    },
});
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const HudContext = React.createContext({
    hide: () => undefined,
    show: () => undefined,
});
export const useHudContext = () => {
    const context = useContext(HudContext);
    return context;
};
export const HudProvider = (_a) => {
    var { IconComponent, backgroundColor = 'rgba(0,0,0,0.8)', borderRadius = 5, children, color = 'white', containerStyles, fadeDuration = 700, height = 80, rotate = false, rotateDuration = 800, style, useNativeDriver = true, width = 80 } = _a, viewProps = __rest(_a, ["IconComponent", "backgroundColor", "borderRadius", "children", "color", "containerStyles", "fadeDuration", "height", "rotate", "rotateDuration", "style", "useNativeDriver", "width"]);
    const fadeAnimationRef = useRef(new Animated.Value(0));
    const rotationAnimationRef = useRef(new Animated.Value(0));
    const [iconProps, setIconProps] = useState();
    const defaultOptions = useMemo(() => ({
        backgroundColor,
        borderRadius,
        color,
        fadeDuration,
        height,
        rotate,
        rotateDuration,
        width,
    }), [
        backgroundColor,
        borderRadius,
        color,
        fadeDuration,
        height,
        rotate,
        rotateDuration,
        width,
    ]);
    const [options, setOptions] = useState(defaultOptions);
    const fadeIn = useCallback(() => {
        Animated.timing(fadeAnimationRef.current, {
            duration: options.fadeDuration,
            toValue: 1,
            useNativeDriver,
        }).start();
    }, [options.fadeDuration, useNativeDriver]);
    const fadeOut = useCallback(() => {
        Animated.timing(fadeAnimationRef.current, {
            duration: options.fadeDuration,
            toValue: 0,
            useNativeDriver,
        }).start(({ finished }) => {
            if (finished) {
                rotationAnimationRef.current.stopAnimation();
                setIconProps(undefined);
            }
        });
    }, [options.fadeDuration, useNativeDriver]);
    const startRotating = useCallback(() => {
        rotationAnimationRef.current.setValue(0);
        Animated.loop(Animated.timing(rotationAnimationRef.current, {
            duration: options.rotateDuration,
            easing: Easing.linear,
            toValue: 1,
            useNativeDriver,
        })).start();
    }, [options.rotateDuration, useNativeDriver]);
    const show = useCallback((nextIconProps, incomingOptions) => {
        fadeIn();
        const filteredIncomingOptions = incomingOptions
            ? Object.fromEntries(Object.entries(incomingOptions).filter(([, value]) => value !== undefined))
            : undefined;
        const nextOptions = Object.assign(Object.assign({}, defaultOptions), filteredIncomingOptions);
        setOptions(nextOptions);
        if (nextOptions.rotate) {
            startRotating();
        }
        setIconProps(Object.assign({ color: defaultOptions.color, size: Math.min(nextOptions.width, nextOptions.height) * 0.5 }, nextIconProps));
    }, [defaultOptions, fadeIn, startRotating]);
    const hide = useCallback(() => {
        fadeOut();
    }, [fadeOut]);
    const iconContainerStyles = useMemo(() => [
        styles.iconContainer,
        {
            backgroundColor: options.backgroundColor,
            borderRadius: options.borderRadius,
            height: options.height,
            opacity: fadeAnimationRef.current,
            width: options.width,
        },
        containerStyles,
    ], [
        containerStyles,
        options.backgroundColor,
        options.borderRadius,
        options.height,
        options.width,
    ]);
    const iconWrapperStyles = useMemo(() => (options === null || options === void 0 ? void 0 : options.rotate)
        ? {
            transform: [
                {
                    rotate: rotationAnimationRef.current.interpolate({
                        inputRange: [0, 1],
                        outputRange: ['0deg', '360deg'],
                    }),
                },
            ],
        }
        : {}, [options === null || options === void 0 ? void 0 : options.rotate]);
    return (<HudContext.Provider value={{ hide, show }}>
      <View style={StyleSheet.flatten([styles.view, style])} testID="hud-view" {...viewProps}>
        {children}
        {iconProps ? (<View pointerEvents="none" style={styles.container} testID="hud-container">
            <Animated.View style={iconContainerStyles} testID="hud-icon-container">
              <Animated.View style={iconWrapperStyles} testID="hud-icon-wrapper">
                <IconComponent {...iconProps}/>
              </Animated.View>
            </Animated.View>
          </View>) : null}
      </View>
    </HudContext.Provider>);
};
